/*
 * ################################################################
 *
 *                    Fractal GCM Management API
 *
 * Copyright (C) 2009 INRIA, University of
 *                    Nice-Sophia Antipolis, ActiveEon
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: proactive@ow2.org
 *
 * Authors: INRIA, University of Nice-Sophia Antipolis, ActiveEon.
 *
 * ################################################################
 */
package org.etsi.uri.gcm.api.control;

import java.util.List;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * Component interface to control gathercast interfaces of the component to
 * which it belongs, notably bindings on gathercast interfaces. This interface
 * is an extension of the {@link CollectiveInterfaceController} interface.
 * 
 * @author INRIA, University of Nice-Sophia Antipolis, ActiveEon
 * @see CollectiveInterfaceController
 */
public interface GathercastController extends CollectiveInterfaceController {

    /**
     * Notifies the component to which this interface belongs that a binding has
     * been performed to one of its gathercast interfaces.
     * 
     * @param gathercastItfName
     *            Name of a gathercast interface of the component to which this
     *            interface belongs.
     * @param owner
     *            Reference on the component connecting to the gathercast
     *            interface.
     * @param clientItfName
     *            Name of the interface connecting to the gathercast interface.
     * @throws NoSuchInterfaceException
     *             If there is no such gathercast or client interface.
     * @throws IllegalLifeCycleException
     *             If the component to which this interface belongs is not in an
     *             appropriate state to perform this operation.
     * @throws IllegalBindingException
     *             If an error related to binding occurs.
     */
    public void notifyAddedGCMBinding(String gathercastItfName,
                                      Component owner, String clientItfName)
            throws NoSuchInterfaceException, IllegalBindingException,
            IllegalLifeCycleException;

    /**
     * Notifies the component to which this interface belongs that a binding has
     * been removed from one of its gathercast interfaces.
     * 
     * @param gathercastItfName
     *            Name of a gathercast interface of the component to which this
     *            interface belongs.
     * @param owner
     *            Reference on the component connected to the gathercast
     *            interface.
     * @param clientItfName
     *            Name of the interface connected to the gathercast interface.
     * @throws NoSuchInterfaceException
     *             If there is no such gathercast or client interface.
     * @throws IllegalLifeCycleException
     *             If the component to which this interface belongs is not in an
     *             appropriate state to perform this operation.
     * @throws IllegalBindingException
     *             If an error related to binding occurs.
     */
    public void notifyRemovedGCMBinding(String gathercastItfName,
                                        Component owner, String clientItfName)
            throws NoSuchInterfaceException, IllegalBindingException,
            IllegalLifeCycleException;

    /**
     * Returns a list of references on the interfaces connected to a given
     * gathercast interface of the component to which this interface belongs.
     * 
     * @param gathercastItfName
     *            Name of a gathercast interface of the component to which this
     *            interface belongs.
     * @return List of interfaces connected to this gathercast interface, or
     *         <tt>null</tt> if it is not bound.
     * @throws NoSuchInterfaceException
     *             If there is no such gathercast interface.
     */
    public List<Object> getGCMConnectedClients(String gathercastItfName)
            throws NoSuchInterfaceException;
}
