/*
 * ################################################################
 *
 *                    Fractal GCM Management API
 *
 * Copyright (C) 2009 INRIA, University of
 *                    Nice-Sophia Antipolis, ActiveEon
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: proactive@ow2.org
 *
 * Authors: INRIA, University of Nice-Sophia Antipolis, ActiveEon.
 *
 * ################################################################
 */
package org.etsi.uri.gcm.api.control;

import java.util.Map;

import org.objectweb.fractal.api.NoSuchInterfaceException;

/**
 * Component interface to control the monitoring of the component to which it
 * belongs. This controller manages statistics for each method exposed by the
 * server interfaces of the component to which this interface belongs.
 * 
 * @author INRIA, University of Nice-Sophia Antipolis, ActiveEon
 */
public interface MonitorController {

    /**
     * Starts the monitoring of the component to which this interface belongs.
     */
    public void startGCMMonitoring();

    /**
     * Stops the monitoring of the component to which this interface belongs.
     */
    public void stopGCMMonitoring();

    /**
     * Resets the monitoring of the component to which this interface belongs.
     * All collected statistics are deleted.
     */
    public void resetGCMMonitoring();

    /**
     * Checks if the monitoring of the component to which this interface belongs
     * is started.
     * 
     * @return <tt>true</tt> if the monitoring is started, <tt>false</tt>
     *         otherwise.
     */
    public boolean isGCMMonitoringStarted();

    /**
     * Returns the statistics of a method exposed by a server interface of the
     * component to which this interface belongs.
     * 
     * @param itfName
     *            Name of a server interface of the component to which this
     *            interface belongs.
     * @param methodName
     *            Name of a method exposed by the server interface corresponding
     *            to the given server interface name.
     * @param parameterTypes
     *            Parameter types of the method corresponding to the given
     *            method name.
     * @return Object containing all the statistics of this method.
     * @throws NoSuchInterfaceException
     *             If there is no such server interface.
     * @throws NoSuchMethodException
     *             If there is no such method.
     */
    public Object getGCMStatistics(String itfName, String methodName,
                                   Class<?>[] parameterTypes)
            throws NoSuchInterfaceException, NoSuchMethodException;

    /**
     * Returns the statistics for each method exposed by each server interface
     * of the component to which this interface belongs.
     * 
     * @return Statistics for each method exposed by each server interface of
     *         the component to which this interface belongs. The statistics are
     *         provided through a {@link Map} object where the key is a single
     *         ID for each method and allows to get an object containing the
     *         statistics of the method corresponding to this ID.
     */
    public Map<String, Object> getAllGCMStatistics();
}
